//SCORM LMS API implementation
//author: Andrei Borziak

//AB 2008-Jun-03
var lms_merge = 1;		//merge putparam and putinteractions
var lms_stid = 0;		//test id

var bAlert = false;			//set false to avoid the alert boxes
var initLMS = -2;			//-2 - never loaded with LMS data
var sErrorCode = "0";		//error code
var sStatus = "not attempted";	//lesson (course) status
var oScore = new Array(0,100,0);//last test score, max score, min score
var sLocation = "0";		//bookmark
var sSuspendData = "";		//suspend data = progress
var sExitStatus = "";		//exit status
var sLessonMode = "";		//lesson mode
var iMasteryScore = 100;	//The Mastery score for this learning object
var sStudentName = "";		//student name
var total_time = 0;			//total time in course in sec.
var interactions = new Array();	//array for interactions

var API = new APIobj();		//API object is created

function APIobj()
{
	this.LMSInitialize = LMSInitialize;
	this.LMSGetValue = LMSGetValue;
	this.LMSSetValue = LMSSetValue;
	this.LMSCommit = LMSCommit;
	this.LMSGetLastError = LMSGetLastError;
	this.LMSGetErrorString = LMSGetErrorString;
	this.LMSGetDiagnostic = LMSGetDiagnostic;
	this.LMSFinish = LMSFinish;
	this.init = init;
}

function LMSInitialize(param) {
	if( initLMS <= -2 )
	{
		if( bAlert ) alert("API: LMSInitialize failed.");
		return false;
	}
	initLMS = 1;
	if( bAlert ) alert("API: LMSInitialize succeded."); 
	return true; 
}

function LMSGetValue(element) {
	if( initLMS <= 0 )
	{
		sErrorCode = "301";	// not initialized
		if( bAlert ) alert("API: LMSGetValue - not initialized: " + element );
		return "";
	}
	sErrorCode = "0";

	switch (element) {
		case "cmi.interactions._children":		return "id,student_response";
		case "cmi.interactions._count":			return interactions.length;

		case "cmi.core.lesson_status":			return sStatus;
		case "cmi.core.score._children":		return "raw,max,min";
		case "cmi.core.score.raw":		 		return oScore[0];
		case "cmi.core.score.max":		 		return oScore[1];
		case "cmi.core.score.min":		 		return oScore[2];
		case "cmi.core.total_time":		 		return getTotalTime();
		case "cmi.core.lesson_location":		return sLocation;
		case "cmi.suspend_data":				return sSuspendData;
	
		case "cmi.student_data.mastery_score":  return iMasteryScore;
		case "cmi.student_data.max_time_allowed":  return "00:15:00";
		case "cmi.core.student_name":			return sStudentName;
		case "cmi.core.student_id":				return "";
		case "cmi.core._children":				return "student_name,student_id,lesson_status,lesson_location,score,total_time,session_time,suspend_data,credit,launch_data,entry,exit";
		case "cmi.core.credit":					return "credit";
		case "cmi.core.entry":					return "";
		case "cmi.core.exit":					return sExitStatus;
		case "cmi.core.launch_data":			return "";
		case "cmi.core.lesson_mode":			return sLessonMode;

		default: 
			if( bAlert ) alert("API: LMSGetvalue: \t\t" + element + "\n\n not catered for: returning:\t" + element); 
			sErrorCode = "401";	// not implemented
	}
	return "";
}

function LMSSetValue(element,value) {
//alert( element + "," + value )
	if( initLMS <= 0 )
	{
		if( bAlert ) alert("API: LMSSetValue - not initialized: " + element );
		sErrorCode = "301";	// not initialized
		return "";
	}
	sErrorCode = "0";

	switch (element) {
		case "cmi.core.lesson_status":		sStatus = value; return true;
		case "cmi.core.score.raw":			return updateScore( value );
		case "cmi.core.score.max":		 	oScore[1] = value; return true;
		case "cmi.core.score.min":		 	oScore[2] = value; return true;
		case "cmi.core.lesson_location":	sLocation = value; return true;
		case "cmi.core.session_time":		saveTime(value); return true;
		case "cmi.core.exit":				sExitStatus = value; return true;
		case "cmi.suspend_data": 			sSuspendData = value; return true;
		case "cmi.core.launch_data":		lms_stid = value; return true;	//AB 2008-Jun-03

		default: 
			var ss = element.split( "." ), i;
			if( ss.length == 4 && ss[ 0 ] == "cmi" && ss[ 1 ] == "interactions" && 
				( ss[ 3 ] == "id" || ss[ 3 ] == "student_response" ) )
			{
				i = new Number( ss[ 2 ] );
				if( !(isNaN( i ) || i < 0 ) )
				{
					if( i >= interactions.length || interactions[ i ] == null )
						interactions[ i ] = new GT_interaction();
				
					if( ss[ 3 ] == "id" )
						interactions[i].id = value;
					else if( ss[ 3 ] == "student_response" )
						interactions[i].response = value;
					return true;
				}
			}
				
			if( bAlert ) alert("API: LMSSetvalue: \t\t" + element + "\n\n This value will not be saved:\t" + value);
			sErrorCode = "401";	// not implemented
	}
	return false;
}

function LMSCommit() {
	if( initLMS <= 0 )
	{
		if( bAlert ) alert("API: LMSCommit - not initialized: " );
		sErrorCode = "301";	// not initialized
		return "";
	}
	sErrorCode = "0";
	calcStatusAndSuspendData();
	
	//trace
	var i, s = "API: LMSCommit called\nValues tracked and saved";
	for( i = 0; i < interactions.length; i++ )
	{
		if( interactions[ i ] == null ) continue;
		s +=  "\ncmi.interactions." + i +":\t" 	+ interactions[i].id + ", " + interactions[i].response;
	}
	s +=  "\ncmi.core.score:\t\t"			+ oScore[0];
	s +=  "\ncmi.core.total_time:\t"		+ getTotalTime();
	s +=  "\ncmi.core.lesson_status:\t"		+ sStatus;
	s +=  "\ncmi.core.lesson_location:\t"	+ sLocation;
	s +=  "\ncmi.core.suspend_data:\t"		+ sSuspendData;
	if( bAlert ) alert(s);
	
	for( s = "", i = 0; i < interactions.length; i++ )
	{
		if( interactions[ i ] == null ) continue;
		if( s == "" ) s += ( '"interaction_id","student_response"' + "\r\n" );
		s +=  interactions[i].id + "," + interactions[i].response + "\r\n";
	}
	
	//commit core data, addCommand in player.htm
	if( lms_merge == 0 || s == "" ) addCommand1( "putparam", getCore() + "[Core_Lesson]\r\n" + sSuspendData );

	if( s != "" )
	{	//commit interactions
		if( lms_merge == 0 ) {
			addCommand1( "putinteractions", s );
		} else {
			var stid_s = ( oScore[ 0 ] >= iMasteryScore ) ? "p" : "i";
			if( typeof(pages) == "undefined" || pages == null ) {}
			else { stid_s = pages[ curPage ].stat; }
	
			s = getCore() + "[Core_Lesson]\r\n" + sSuspendData + "\r\nSTID=" + lms_stid + "\r\nSTID_S=" + stid_s + "\r\n[putinteractions]\r\n" + s;
//			if( lms_stid != 0 ) alert( s );
			addCommand1( "putparam", s );
		}
	}
	interactions = new Array();		//clean interactions
	
	return true;
}

function LMSFinish() {
	if( bAlert ) alert("API: LMSFinish called");
	if( initLMS <= 0 )
	{
		if( bAlert ) alert("API: LMSFinish - not initialized: " );
		sErrorCode = "301";	// not initialized
		return "";
	}
	//??LMSCommit();
	sErrorCode = "0";
	initLMS = -1;
	return true;
}

function LMSGetLastError() { 
	if( bAlert ) alert("API: LMSGetLastError called: " + sErrorCode); 
	var err = sErrorCode;
	sErrorCode = "0";
	return err;
}

function LMSGetErrorString( err ) { 
	if( bAlert ) alert("API: LMSGetErrorString called: " + err);
	var i = err + 0;
	switch( i ) {
		case 0: return "No error"; 
		case 301:	return "Not initialized";
		case 401: return "Not implemented";
	}
	return "Error code unknown" ;
}

function LMSGetDiagnostic() { 
	if( bAlert ) alert("API: LMSGetDiagnostic called");
	return "No error tracking in this api";
}

function getTotalTime() {
	var dtTime = new Date("01/01/2002");
	dtTime.setTime( dtTime.getTime() + total_time * 1000 );
	var strTime = "";
	var h = dtTime.getHours();
	var m = dtTime.getMinutes();
	var s = dtTime.getSeconds();

	if(h<10) strTime+="0";
	strTime+= h + ":";
	if(m<10) strTime+="0";
	strTime+= m + ":";
	if(s<10) strTime+="0";
	strTime+= s;
//alert( "getTotalTime()=" + strTime );
	return strTime;
}

function saveTime(time) {
	var t = time.split(":");
	var dtSessionTime = t[0] * 3600 + t[1] * 60 + t[2] * 1;

	// Update total Time
	total_time += dtSessionTime;
//alert( "saveTime(),time=" + time + ",total=" + total_time );
}

function GT_interaction(id, response) {
	this.id = (!id) ? "0" : id;
	this.response = (!response) ? "0" : response;
}

function getCore()
{
	var	core	= "[core]\r\n";
	core	+= "lesson_status=" + sStatus + "\r\n";
	core	+= "score=" + oScore[ 0 ] + "\r\n";
	core	+= "lesson_location=" + sLocation + "\r\n";
	core	+= "time=" + getTotalTime() + "\r\n";
	return core;
}

function init( aiccStatus, aiccScore, aiccLocation, aiccSuspendData, 
	aiccLessonMode, aiccMasteryScore, aiccStudentName, aiccTotal_time )
{
	if( bAlert ) alert( "API: init " + aiccStudentName );
	initLMS = -1;	//ready for LMSInitialize
	sStatus = aiccStatus;
	oScore[ 0 ] = 1 * aiccScore;
	sLocation = 1 * aiccLocation;
	sSuspendData = aiccSuspendData;
	sLessonMode = aiccLessonMode;
	iMasteryScore = 1 * aiccMasteryScore;
	sStudentName = aiccStudentName;
	saveTime( aiccTotal_time );
	parseSuspendData();
	displaylessonstatus();
}
//pages[], curPage are in "player.htm"
function updateScore( value )
{
	if( typeof(pages) == "undefined" || pages == null ) return false;
	
	var result = value >= iMasteryScore;
	if( result )
	{
		pages[ curPage ].stat = "p";	//passed
		oScore[0] = value; 
	}
	else if( pages[ curPage ].stat != "p" )
	{	//count only first try
		pages[ curPage ].stat = "f";	//failed
		oScore[0] = value; 
	}
	if( pages[ curPage ].iScore < value ) pages[ curPage ].iScore = value;
	return true;
}

function calcStatusAndSuspendData()
{
	if( typeof(pages) == "undefined" || pages == null ) return;
	
	var i, tmp = "", tmpStatus = "passed";
	for( i = 0; i < pages.length; i++ )
	{
		tmp = tmp + (i + 1) + ":" + pages[ i ].stat;
		if( pages[ i ].iScore >= 0 ) tmp = tmp + ":" + pages[ i ].iScore;
		tmp = tmp + ",";
		if( ( pages[ i ].stat == "i" || pages[ i ].stat == "f" ) &&
			pages[ i ].url != "" ) 
			tmpStatus = "incomplete";
	}
	sSuspendData = tmp;
	var s = sStatus.toLowerCase();
	if( s != "passed" && s != "completed" )
	{
		sStatus = tmpStatus;	//override
		if( sStatus == "passed" ) 
		{	//notify parent
			var op = window.parent.opener;
			if( op != null && op.myfresh != null ) op.myfresh();
		}
	}
	displaylessonstatus();
}

function parseSuspendData()
{
	var i, stat, iDone = false;
	var s = sStatus.toLowerCase();
	if( s == "passed" || s == "completed" )
	{	//ignore suspend_data
		for( i = 0; i < pages.length; i++ )
		{
			pages[ i ].stat = "p";
		}
		iDone = true;
	}

	if( sSuspendData == "" ) return;

	var pairs = sSuspendData.split(",");
	for( i = 0; i < pages.length && i < pairs.length; i++ )
	{
		stat = pairs[ i ].split( ":" );
		if( stat.length >= 2 && !iDone )
			pages[ stat[ 0 ] - 1 ].stat = stat[ 1 ];
		if( stat.length >= 3 )
			pages[ stat[ 0 ] - 1 ].iScore = stat[ 2 ];
	}
}

function addCommand1( key, val )
{
	if( typeof(pages) == "undefined" || pages == null ) return;
	
	addCommand( key, val );	//in "player.htm"
}